% TGRShis program is used to retrieve sea surface wind speed from the
% Sentinel-1 HH polarization data using the developed BP neural network
% Reference: Retrieval of sea surface wind speed from spaceborne SAR over the Arctic marginal ice zone with a neural network
% Submitted to IEEE TGRS by Xiao-Ming Li, Tingting Qin and Ke Wu

% The program was created by: Ms. Ke Wu
% Affiliation: Aerospace Information Research Institution
%              Chinese Academy of Sciences
% email: wk721169@outlook.com
% Date of creation: 15th, May, 2020

% The function read_Sentinel_kml is modified based on the function
% reak_kml which is downloaded at MATLAB Central and written by afarris@usgs.gov 2006 November


clear
clc
%% User input
%please input Sentinel-1 images
[Filename,File_path]=uigetfile('*.tiff','please choose HH-polarization SAR files');
vfilen=[File_path,Filename];

if ~exist(vfilen)
    error('the file does not exist!');
end

% read the Sentinel-1 images and XML files
Image=double(imread(vfilen));

idst=strfind(vfilen,'\');
S_filepath=vfilen(1:(max(idst)-12));
S_filename=vfilen((max(idst)+1):end-4);

xmln_HH=[S_filepath,'annotation\',S_filename,'xml'];                         % read Sentinel-1 annotation  files
[meta,~]= xml_read(xmln_HH);
xml_c_HH=[S_filepath,'annotation\calibration\calibration-',S_filename,'xml'];% read Sentinel-1 calibration files
[calibration_tree,~]=xml_read(xml_c_HH);
xml_n_HH=[S_filepath,'annotation\calibration\noise-',S_filename,'xml'];      % read Sentinel-1 noise files
[noise_tree,~]=xml_read(xml_n_HH);
clear Filename  File_path xmln_HH xml_c_HH xml_n_HH

%% Radiometric Calibration 
% read and interpolate calibration vector
[xf1,yf1]=size(Image);  
b2=1;
cali=zeros;
ll=length(calibration_tree.calibrationVectorList.calibrationVector);
for ii=1:ll    
    if ii==1
        co=calibration_tree.calibrationVectorList.calibrationVector(ii).sigmaNought.ATTRIBUTE.count;
        cali(b2,1:co)=calibration_tree.calibrationVectorList.calibrationVector(ii).sigmaNought.CONTENT;        
        hh1=calibration_tree.calibrationVectorList.calibrationVector(ii).line;      
    else
        hh=calibration_tree.calibrationVectorList.calibrationVector(ii).line;
        if hh==hh1
            cali(b2,1:co)=calibration_tree.calibrationVectorList.calibrationVector(ii).sigmaNought.CONTENT;            
            hh1=calibration_tree.calibrationVectorList.calibrationVector(ii).line;           
        else
            b2=b2+1;           
            cali(b2,1:co)=calibration_tree.calibrationVectorList.calibrationVector(ii).sigmaNought.CONTENT;           
            hh1=calibration_tree.calibrationVectorList.calibrationVector(ii).line;
        end
    end
end
[rows,cols]=size(cali);
[cx,cy]=meshgrid(1:1:rows,1:1:cols);
rowss=xf1;colss=yf1;
[cxx,cyy]=meshgrid(1:((rows-1)/(rowss-1)):rows,1:((cols-1)/(colss-1)):cols);
cali1=interp2(cx,cy,cali',cxx,cyy,'cubic');
cali=cali1';
clear  b1 b2 ll ii hh hh1 co cxx cyy cx cy rows cols rowss colss cali1

% read and interpolate thermal noise
b2=1; no=zeros;
time=str2num(S_filename(15:22));
if time < 20180313          
% If the IPF version priori to the version 2.9,only the noise vector in range 
% direction is provided by ESA
    ll=length(noise_tree.noiseVectorList.noiseVector);
    for ii=1:ll    
        if ii==1
            co=noise_tree.noiseVectorList.noiseVector(ii).noiseLut.ATTRIBUTE.count;
            no(b2,1:co)=noise_tree.noiseVectorList.noiseVector(ii).noiseLut.CONTENT;        
            hh1=noise_tree.noiseVectorList.noiseVector(ii).line;        
        else
            co=noise_tree.noiseVectorList.noiseVector(ii).noiseLut.ATTRIBUTE.count;
            hh=noise_tree.noiseVectorList.noiseVector(ii).line;
            if hh==hh1
                no(b2,1:co)=noise_tree.noiseVectorList.noiseVector(ii).noiseLut.CONTENT;            
                hh1=noise_tree.noiseVectorList.noiseVector(ii).line;            
            else
                b2=b2+1;           
                no(b2,1:co)=noise_tree.noiseVectorList.noiseVector(ii).noiseLut.CONTENT;            
                hh1=noise_tree.noiseVectorList.noiseVector(ii).line;           
            end
        end
    end
else                                                                      
%For the IPF version after the version 2.9(available from Mar.13, 2018)
%The noise vectors in both range and azimuth direction are available.
    leng=length(noise_tree.noiseRangeVectorList.noiseRangeVector);
    RangeXn=zeros();RangeYn=zeros();sigma0n1=zeros();
    for ii=1:leng
        RangeXn(ii)=noise_tree.noiseRangeVectorList.noiseRangeVector(ii).line+1;
        RangeYnn=noise_tree.noiseRangeVectorList.noiseRangeVector(ii).pixel;
        RangeYn(ii,1:RangeYnn.ATTRIBUTE.count)=RangeYnn.CONTENT+1;
        sigma011=noise_tree.noiseRangeVectorList.noiseRangeVector(ii).noiseRangeLut;   
        sigma0n1(ii,1:RangeYnn.ATTRIBUTE.count)=sigma011.CONTENT;
    end
    
    %process range noise vectors at the frames around the image
    [~,y]=find(sigma0n1==0);
    if isempty(y)==0                                                       %if noise vector has 0 value
        for i=2:length(y) 
            if y(i)~=y(i-1)
                sigma0n1oneline=sigma0n1(:,y(i-1));
                index=find(sigma0n1oneline==0);                            %find the position of 0 value and remove them
                sigma0n1oneline(index)=[];
                xx=RangeXn;
                xx2=xx(index);
                xx(index)=[];
                if length(sigma0n1oneline)>10
                    sigma0n1(index,y(i-1))=interp1(xx,sigma0n1oneline,xx2,'pchip');
                end
            end
        end
    end

    %interpolate range noise and azimuth noise
    clear('azNoise','azNoise11');
    leng=length(noise_tree.noiseAzimuthVectorList.noiseAzimuthVector);
    no=zeros();  no2=zeros();
    %calculate the complete noise in each sub-swath
    for ii=1:leng 
        clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
        firstAzimuthLine=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstAzimuthLine+1;
        firstRangeSample=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).firstRangeSample+1;
        lastAzimuthLine=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastAzimuthLine+1;
        lastRangeSample=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).lastRangeSample+1; 
        clear('x','x1','x2','index1','index2','index11','index22','index3','index4');
        
        %find the closest azimuth line in calibrating matrix to the first azimuth line in subswath
        [x1,index11]=sort(abs(RangeXn-firstAzimuthLine));                 
        if RangeXn(index11(1))<=firstAzimuthLine
            index1=index11(1);
        else
            index1=index11(1)-1;
        end
        
        %find the closest azimuth line in calibrating matrix to the last azimuth line in subswath
        [x2,index22]=sort(abs(RangeXn-lastAzimuthLine));                  
        if RangeXn(index22(1))<lastAzimuthLine
            index2=index22(1)+1;
        else
            index2=index22(1);
        end
        
        %if the azimuth width is less than 1 pixel, then the range vector is the 1-D interpolation of the closest range vector
        if index2-index1<=1 
            if x1(1)<x2(1)
                midAzsubsw=index11(1);
            else
                midAzsubsw=index22(1);
            end
            clear('index3','index4','temp','rrsam');
            
            [~,index3]=sort(abs(RangeYn(midAzsubsw,:)-firstRangeSample));
            [~,index4]=sort(abs(RangeYn(midAzsubsw,:)-lastRangeSample));
            rrsam=firstRangeSample:1:lastRangeSample;
            temp=interp1(RangeYn(midAzsubsw,index3(1):index4(1)),sigma0n1(midAzsubsw,index3(1):index4(1)),rrsam,'pchip');          
            for jj=1:lastAzimuthLine-firstAzimuthLine+1
               rsNoise1(jj,:)=temp;                                     
            end
        
        %if the azimuth width is equal to 1 pixel, then the range vector is the 1-D interpolation of the closest range vector
        elseif index2-index1==2  
            clear('index3','index4');
            midAzsubsw=ceil((index1+index2)/2);
            index3=find(RangeYn(midAzsubsw,:)==firstRangeSample);
            index4=find(RangeYn(midAzsubsw,:)==lastRangeSample);

            nx=RangeXn(index1:index2);ny=RangeYn(midAzsubsw,index3:index4);
            [nY,nX]=meshgrid(ny,nx);
            nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=firstRangeSample:1:lastRangeSample;
            [nYY,nXX]=meshgrid(nyy,nxx);
            rsNoise1=griddata(nX,nY,sigma0n1(index1:index2,index3:index4),nXX,nYY,'cubic');
        
        %if the azimuth width is more than 1 pixel
        else
            clear('index3','index4','temp','sigma0n2','nx','ny','nxx','nyy','nX','nY','nXX','nYY');
            midAzsubsw=ceil((index1+index2)/2);
            index3=find(RangeYn(midAzsubsw,:)==firstRangeSample);
            index4=find(RangeYn(midAzsubsw,:)==lastRangeSample);
            sigma0n2=zeros(size(sigma0n1(index1:index2,index3:index4)));
            
            for mm=index3:index4
                nx=zeros(1,length(RangeXn(index1:index2)));
                nx(1,1)=firstAzimuthLine;
                nx(1,2:end-1)=RangeXn(index1+1:index2-1);
                nx(1,end)=lastAzimuthLine;
                temp=interp1(nx(1,2:end-1),sigma0n1(index1+1:index2-1,mm)',nx,'pchip');
                sigma0n2(:,mm-index3+1)=temp';
            end
            
            ny=RangeYn(midAzsubsw,index3:index4);
            [nY,nX]=meshgrid(ny,nx);
            nxx=firstAzimuthLine:1:lastAzimuthLine;nyy=firstRangeSample:1:lastRangeSample;
            [nYY,nXX]=meshgrid(nyy,nxx);
            rsNoise1=griddata(nX,nY,sigma0n2,nXX,nYY,'cubic');    
        end
        
        %interpolate the azimuth noise
        azLine=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).line;
        noiseAzimuthLut=noise_tree.noiseAzimuthVectorList.noiseAzimuthVector(ii).noiseAzimuthLut;
        noiseAzimuthLutval=noiseAzimuthLut.CONTENT;
        
        if ischar(noiseAzimuthLutval)
            noiseAzimuthLutval2=zeros();
            index1=zeros(1,azLine.ATTRIBUTE.count+1);
            index1(2:end-1)=find(isspace(noiseAzimuthLutval));
            index1(1)=0;index1(end)=length(noiseAzimuthLutval)+1;
            for jj2=1:length(index1)-1
                noiseAzimuthLutval2(jj2)=str2double(noiseAzimuthLutval(index1(jj2)+1:index1(jj2+1)-1));
            end
            noiseAzimuthLutval=noiseAzimuthLutval2;
        end
        
        azline2=azLine.CONTENT;
        if ischar(azline2)
            azline2temp=zeros();
            index1=zeros(1,azLine.ATTRIBUTE.count+1);
            index1(2:end-1)=find(isspace(azline2));
            index1(1)=0;index1(end)=length(azline2)+1;
            for jj2=1:length(index1)-1
                azline2temp(jj2)=str2double(azline2(index1(jj2)+1:index1(jj2+1)-1));
            end
            azline2=azline2temp;
        end
        
        %if the rows number of subswath is equal to 1, then all the subswath value are same
        if azLine.ATTRIBUTE.count==1 
            azNoise1=noiseAzimuthLutval;
        else                                                               %the azimuth noise is 2-D interpolation of the azimuth noise vector
            azxx=firstAzimuthLine:1:lastAzimuthLine;
            azNoise11=interp1(azline2,noiseAzimuthLutval,azxx,'pchip');
            swathWide=lastRangeSample-firstRangeSample+1;
            azNoise22=repmat(azNoise11,swathWide,1);
            azNoise1=azNoise22';
        end
        
        noo=azNoise1.*rsNoise1;                                            %thermal noise is equal to azimuth noise multiply to range noise 
        no(firstAzimuthLine:lastAzimuthLine,firstRangeSample:lastRangeSample)=noo;
        clear azNoise1 rsNoise1 noo
    end
    clear('RangeYnn','sigma011','midAzsubsw','rrsam','temp','nx','ny','nxx','nyy','nX','nY','nXX','nYY','azLine','noiseAzimuthLut','azNoise11','azNoise22','azxx','swathWide');
    clear('firstAzimuthLine','firstRangeSample','lastAzimuthLine','lastRangeSample');
    clear('x','x1','x2','index1','index2','index11','index22','index3','index4');
end

[rows,cols]=size(no);
if xf1~=rows||yf1~=cols                                                    %resampling thermal noise
    [cx,cy]=meshgrid(1:rows,1:cols);
    rowss=xf1;colss=yf1;
    [cxx,cyy]=meshgrid(1:((rows-1)/(rowss-1)):rows,1:((cols-1)/(colss-1)):cols);
    no1=interp2(cx,cy,no',cxx,cyy,'cubic');
    no=no1';
end
clear  b1 b2 ll ii hh hh1 co cxx cyy cx cy rows cols rowss colss no1

% thermal noise removal
sigma0L_HH=(double(Image.^2)-no)./cali.^2;
sigma0L_HH(sigma0L_HH(:)<0)=0;
clear  no cali calibration_tree noise_tree

%% black edge removal
for ii=1:yf1
    id=find(Image(:,ii)<=20);
    if length(id)/xf1>0.15
        idy=ii;
    else      
        idy=ii;
        break;
    end
 end
for ii=1:xf1
    id=find(Image(ii,:)<=20);
    if length(id)/yf1>0.15
        idx=ii;
    else   
        idx=ii;
        break;
    end
end
for ii=xf1:-1:1
     id=find(Image(ii,:)<=20);
    if length(id)/yf1>0.15
        idxh=ii;
    else
         idxh=ii;
        break;  
    end
end
for ii=yf1:-1:1
     id=find(Image(:,ii)<=20);
    if length(id)/xf1>0.15
        idyh=ii;
    else
        idyh=ii;
        break;
    end
end        
sigma0L_hh=sigma0L_HH(idx:idxh,idy:idyh);
clear id  ii xf1 yf1

%% interpolate latitude, longitude and incident angle matrix
n=1;m=1;
lat=zeros;lon=zeros;rowline=zeros;colpixel=zeros;
ll=length(meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint);
%read the latitude, longitude and incident angle vector
for ii=1:ll
    lat(n,m)=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).latitude;
    lon(n,m)=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).longitude;
    inc(n,m)=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).incidenceAngle;
    rowline(n,m)=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).line+1;
    colpixel(n,m)=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii).pixel+1;
    if ii<ll
        if rowline(n,m)~=meta.geolocationGrid.geolocationGridPointList.geolocationGridPoint(ii+1).line+1
            n=n+1;
            m=1;
        else 
            m=m+1;
        end
    end
end

[xf1,yf1]=size(sigma0L_HH);
nxx=1:1:xf1;nyy=1:1:yf1;
[nY,nX]=meshgrid(nyy,nxx);
lat2=griddata(rowline,colpixel,lat,nX,nY,'cubic');                         %interpolate the latitude matrix

%if the longitude crosses 180 degree,then convert it to 0-360 degrees
if max(max(lon))>0 && min(min(lon))<0 && abs(max(max(lon))-180)<5 && abs(min(min(lon))-(-180)<5)
    lon=rem(360+lon,360);
    lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');
else
    lon2=griddata(rowline,colpixel,lon,nX,nY,'cubic');                     %interpolate the longitude matrix
    lon2=rem(360+lon2,360);  
end

%%if the longitude crosses 0 degree,then convert it to -180-180 degrees
if max(max(lon2))>350 && min(min(lon2))<10
    lon2(lon2>180)=lon2(lon2>180)-360;  
end

%interpolate incident angle matrix
[rows,cols]=size(inc);
[cx,cy]=meshgrid(1:1:rows,1:1:cols);
[cxx,cyy]=meshgrid(1:((rows-1)/(xf1-1)):rows,1:((cols-1)/(yf1-1)):cols);
inc1=interp2(cx,cy,inc',cxx,cyy,'cubic');
inc2=inc1';

%remove the black edge of latitude, longitude and incident angle
lat2=lat2(idx:idxh,idy:idyh);
lon2=lon2(idx:idxh,idy:idyh);
inc2=inc2(idx:idxh,idy:idyh);

clear rowline colpixel xf1 yf1 nX nY nxx nyy lon lat inc inc1 idx idxh idy idyh rows cols cx cxx cy cyy n m ll ii 
clear Image

%% resampling SAR image
width_space=meta.imageAnnotation.imageInformation.rangePixelSpacing;       %read range resolution
height_space=meta.imageAnnotation.imageInformation.azimuthPixelSpacing;    %read azimuth resolution
[height,width]=size(sigma0L_hh);                                           %read SAR image's number of rows and columns

N=2000;                                                                    %set subimage size = 2000*2000 m
range1=floor(N/(height_space));                                            %calculate the pixel numbers of each subimage 
range2=floor(N/(width_space));
scale1=floor(height/range1); 
scale2=floor(width/range2);

lat_resized=imresize(lat2,[scale1,scale2],'bicubic');                      %The latitude matrix is re-sampled to match with each sub-image
lon_resized=imresize(lon2,[scale1,scale2],'bicubic');                      %longitude re-sampling
incidence=imresize(inc2,[scale1,scale2],'bicubic');                        %incident angle re-sampling
clear lat2 lon2 inc2

%% Using the ERA-5 reanalysis wind direction as input to the BP neural network

%% extract wind direction and wind speed of ERA5 according to the position of SAR four corners 

%read ERA5 nc files                        
Filename1=['ERA5',vfilen((max(idst)+15):(length(vfilen)-48)),'.nc'];       %nc files name
File_path1='K:\ERA5_download\';                                            %nc files folder
NCpath=[File_path1,Filename1];
clear Filename1 File_path1

%match ERA5 time layer
time=meta.adsHeader.startTime;                                             %read SAR imaging time
year=str2num(time(1:4));
month=str2num(time(6:7));
day=str2num(time(9:10));
hour=str2num(time(12:13));
minute=str2num(time(15:16));

ectime_zero=(datenum(year,month,day)-datenum(1900,1,1))*24;                %calculate the time difference between the 0 o'clock at SAR imaging time and 01/01/1900
if minute>30                                                                  
    hour=hour+1;
end
ectime=ectime_zero+hour;                                                   %calculate ERA5 time corresponding to SAR imaging time
clear ectime_zero year month day hour minute time

timelayer=ncread(NCpath,'time');                                           %read ERA5 time layer
layer=find(timelayer==ectime);                                             %find matched time layer

%if no matching time layer is found, then find the time layer with the smallest time interval
if isempty(layer)                                                            
    timed=timelayer-ectime;                                                   
    [timedmin,nm]=min(abs(timed(:)));                                           
    ectimematch=timelayer(nm);                                             
    aa=find(timed==timedmin);
    if isempty(aa)
        timedmin=-timedmin;
    end
    aa=find(timed==timedmin);
    layer=aa;                       
end                                  
clear timed aa time timedmin nm ectime ectimematch timelayer

%read the latitude and longitude of the four corners of SAR images
PathRoot=[S_filepath,'preview\map-overlay','.kml'];
[Lat_Line,Lon_Line] = read_Sentinel_kml( PathRoot);                        %read SAR kml files

latbotr=Lat_Line(1,1); latbotl=Lat_Line(1,2);
lattopl=Lat_Line(1,3); lattopr=Lat_Line(1,4);
lonbotr=Lon_Line(1,1); lonbotl=Lon_Line(1,2);
lontopl=Lon_Line(1,3); lontopr=Lon_Line(1,4);

lontopl=rem(360+lontopl,360);                                              %convert longitude to 0-360 degree
lontopr=rem(360+lontopr,360);      
lonbotl=rem(360+lonbotl,360);      
lonbotr=rem(360+lonbotr,360);        
clear PathRoot 

idx=0;
latmm=[lattopl lattopr latbotl latbotr];
lonmm=[lontopl lontopr lonbotl lonbotr];

if max(Lon_Line)*min(Lon_Line)<0                                           %if the SAR image crosses the zero degree meridian, 
    Lon_Line=rem(360+Lon_Line,360);                                        %the splicing method should be used
    if abs(max(Lon_Line)-min(Lon_Line))>60       
        for i=1:4                                                          %calculate the corresponding position of the four corners longitude in ERA5
            if lonmm(i)>=0
                sub_column(i)=round(lonmm(i)/0.25);
            else
                sub_column(i)=1440+round(lonmm(i)/0.25);
            end
        end

        for i=1:4                                                          %calculate the corresponding position of the four corners latitude in ERA5
            sub_row(i)=round((90-latmm(i))/0.25)+1;
        end

        %extend 2 pixels outwards considering of the edge
        a=min(sub_row)-2;                                                  %the start rows number of choosing subimage
        b=max(sub_row)+2;                                                  %the end rows number of choosing subimage
        c=max(sub_column(find(sub_column<720)))+2;                         %the end columns number of choosing subimage
        d=min(sub_column(find(sub_column>720)))-2;                         %the start columns number of choosing subimage
        if c==0
            c=1;
        end
        clear sub_column sub_row i lonmm latmm

        %extract the u component
        u10n =double(ncread(NCpath,'u10n'));                               %read u component of wind field
        u0_a=u10n(d:1440,a:b,layer);                                       %extract subimage of u component
        u_a=flipud(rot90(u0_a));                                           %flip up and down and rotate 90 degrees counterclockwise
        u0_b=u10n(1:c,a:b,layer);       
        u_b=flipud(rot90(u0_b));   
        u=[u_a,u_b];
        clear u10n u0_a u0_b u_a u_b
        
        %extract the v component
        v10n = double(ncread(NCpath,'v10n'));                              %read v component of wind field
        v0_a=v10n(d:1440,a:b,layer);                                       %extract subimage of v component
        v_a=flipud(rot90(v0_a));                                           %flip up and down and rotate 90 degrees counterclockwise
        v0_b=v10n(1:c,a:b,layer);       
        v_b=flipud(rot90(v0_b));   
        v=[v_a,v_b];
        clear v10n v0_a v0_b v_a v_b           
        clear tmp layer
        
        %extract latitude and longitude in ERA5
        lon_nc=double(ncread(NCpath,'longitude'));                         %read longitude in ERA5
        lat_nc=double(ncread(NCpath,'latitude'));                          %read latitude in ERA5
        lat1=repmat(lat_nc,1,1440);                                        %extend latitude vector to matrix
        lon1=repmat(lon_nc',721,1);                                        %extend longitude vector to matrix
        lat_era5_a=lat1(a:b,d:1440);                                       %extract matched latitude(left part)
        lon_era5_a=lon1(a:b,d:1440);                                       %extract matched longitude(left part)
        lat_era5_b=lat1(a:b,1:c);                                          %extract matched latitude(right part)
        lon_era5_b=lon1(a:b,1:c);                                          %extract matched longitude(right part)
        lon_era5=[lon_era5_a,lon_era5_b];
        lat_era5=[lat_era5_a,lat_era5_b];
        lon_era5( lon_era5>180)= lon_era5( lon_era5>180)-360;              %if the longitude matrix crosses the 0 degree, convert it to -180-180 degree
        clear row col irow jcol lon_nc lat_nc lat1 lon1 lat_era5_a lon_era5_a lat_era5_b lon_era5_b
    else
        idx=1;
    end
else
      idx=1;
end

if idx==1                                                                  %if the SAR image does not cross the zero degree meridian
    for i=1:4                                                              %calculate the corresponding position of the four corners longitude in ERA5
        if lonmm(i)>=0
            sub_column(i)=round(lonmm(i)/0.25);
        else
            sub_column(i)=1440+round(lonmm(i)/0.25);  
        end
    end

    for i=1:4                                                              %calculate the corresponding position of the four corners latitude in ERA5
        sub_row(i)=round((90-latmm(i))/0.25)+1;
    end
    
    a=min(sub_row)-2;                                                      %the start rows number of choosing subimage
    b=max(sub_row)+2;                                                      %the end rows number of choosing subimage
    c=min(sub_column)-2;                                                   %the end columns number of choosing subimage
    d=max(sub_column)+2;                                                   %the start columns number of choosing subimage      
    if c<=0
        c=1;
    end
    if d>1440
        d=1440;
    end
    clear sub_column sub_row i lonmm latmm

    %extract the u component
    u10n =double(ncread(NCpath,'u10n'));                                   %read u component of wind field
    u0=u10n(c:d,a:b,layer);                                                %extract subimage of u component       
    u=flipud(rot90(u0));                                                   %flip up and down and rotate 90 degrees counterclockwise 
    clear u10n u0
    
    %extract the v component
    v10n = double(ncread(NCpath,'v10n'));                                  %read v component of wind field
    v0=v10n(c:d,a:b,layer);                                                %extract subimage of v component
    v=flipud(rot90(v0));                                                   %flip up and down and rotate 90 degrees counterclockwise 
    clear v10n  v0
    clear tmp layer

    %extract latitude and longitude in ERA5
    lon_nc=double(ncread(NCpath,'longitude'));                             %read longitude in ERA5
    lat_nc=double(ncread(NCpath,'latitude'));                              %read latitude in ERA5
    lat1=repmat(lat_nc,1,1440);                                            %extend latitude vector to matrix
    lon1=repmat(lon_nc',721,1);                                            %extend longitude vector to matrix
    lat_era5=lat1(a:b,c:d);                                                %extract matched latitude
    lon_era5=lon1(a:b,c:d);                                                %extract matched longitude
    clear row col irow jcol lon_nc lat_nc lat1 lon1
end     

%% match wind direction and speed for each SAR subimage
model_lon=lon_era5;                                                        %ERA5 latitude
model_lat=lat_era5;                                                        %ERA5 longitude
model_u=u;                                                                 %ERA5 u component
model_v=v;                                                                 %ERA5 v component
space=0.25;                                                                %ERA5 resolution 0.25

[scale1,scale2]=size(lat_resized);
for i=1:scale1    
    for j=1:scale2        
        %calculate the latitude and longitude of the four model points around each subimage
        n=space;                     
        sarlon=lon_resized(i,j);                                           %extract the longitude of each subimage
        sarlat=lat_resized(i,j);                                           %extract the latitude of each subimage
        lat_floor=floor(sarlat/n)*n;                                       %calculate the latitude of ERA5 data corresponding to the SAR subimage(left bottom)
        lon_floor=floor(sarlon/n)*n;                                       %calculate the longitude of ERA5 data corresponding to the SAR subimage(left bottom)
        
        lat_surround=[lat_floor+n lat_floor+n lat_floor lat_floor];        %calculate the latitude of ERA5 data corresponding to the SAR subimage(four points around SAR subimage)
        lon_surround=[lon_floor lon_floor+n lon_floor lon_floor+n];        %calculate the longitude of ERA5 data corresponding to the SAR subimage(four points around SAR subimage)
        
        q=find(lon_surround==360);
        lon_surround(:,q)=359.75;
        clear q
  
        %find the u component corresponding to the four corners
        for k=1:4
            a=find(model_lat==lat_surround(k) & model_lon==lon_surround(k));
            ABCD(k)=model_u(a);
        end
        
        A=ABCD(1);  B=ABCD(2);  C=ABCD(3);  D=ABCD(4);
        Alon=lon_surround(1); Alat=lat_surround(1);
        Blon=lon_surround(2); Blat=lat_surround(2);
        Clon=lon_surround(3); Clat=lat_surround(3);
        Dlon=lon_surround(4); Dlat=lat_surround(4);
        
        %bilinear interpolation 
        Qlon=sarlon;    Qlat=sarlat;
        p1=((B-A)/(Blon-Alon))*(Qlon-Alon)+A;
        p2=((D-C)/(Dlon-Clon))*(Qlon-Clon)+C;
        Q=((p1-p2)/(Alat-Clat))*(Qlat-Clat)+p2;
        
        era5_u(i,j)=Q;
        clear k a A B C D ABCD Alon Alat Blon Blat Clon Clat Dlon Dlat Qlon Qlat p1 p2 Q
        
        %find the v component corresponding to the four corners
        for k=1:4
            a=find(model_lat==lat_surround(k) & model_lon==lon_surround(k));
            ABCD(k)=model_v(a);
        end
        
        A=ABCD(1);  B=ABCD(2);  C=ABCD(3);  D=ABCD(4);
        Alon=lon_surround(1); Alat=lat_surround(1);
        Blon=lon_surround(2); Blat=lat_surround(2);
        Clon=lon_surround(3); Clat=lat_surround(3);
        Dlon=lon_surround(4); Dlat=lat_surround(4);
        
        %bilinear interpolation 
        Qlon=sarlon;    Qlat=sarlat;
        p1=((B-A)/(Blon-Alon))*(Qlon-Alon)+A;
        p2=((D-C)/(Dlon-Clon))*(Qlon-Clon)+C;
        Q=((p1-p2)/(Alat-Clat))*(Qlat-Clat)+p2;
        
        era5_v(i,j)=Q;
        clear k a A B C D ABCD Alon Alat Blon Blat Clon Clat Dlon Dlat Qlon Qlat p1 p2 Q    
    end
end

era5_ws=sqrt(era5_u.*era5_u+era5_v.*era5_v);                               %calculate wind speed ws=sqrt(u.*u+v.*v)
tmp=270.0-atan2(era5_v,era5_u).*180.0/pi;                                  %calculate wind direction
era5_wd=mod(tmp,360.0);                                                    %convert wind direction to 0-360 degree
clear i j m n  space model_lon model_lat model_u model_v lat_floor lat_surround lon_floor lon_surround sarlat sarlon a b c d idx tmp

%calculate mean backscatter coefficient
for m=1:scale1      
    for n=1:scale2  
        ii=(m-1)*range1+1; ii1=m*range1;                                   %calculate rows number of each subimage in SAR image
        jj=(n-1)*range2+1; jj1=n*range2;                                   %calculate columns number of each subimage in SAR image
        
        if ii1>height  
            ii1=height;
        end
        if jj1>width
            jj1=width;
        end
        
        modu=sigma0L_hh(ii:ii1,jj:jj1);                                    %extract subimage in SAR image      
        [L,W]=size(modu);
        
        
        a=find(modu==0);
        if ~isempty(a)                                                     %remove the point that backscatter coefficient is equal to 0
            nm=L*W-size(a,1);
            if nm~=0
                sigma0L_hh2(m,n)=sum(modu(:))/nm;
            else 
                sigma0L_hh2(m,n)=0;   
            end
        else
            sigma0L_hh2(m,n)=mean(modu(:));                                %calculate the mean value
        end
    end
end
clear m n nm ii jj ii1 jj1 modu a L W  height width sigma0L_hh

%calculate look angle
platformheading = meta.generalAnnotation.productInformation.platformHeading;%read SAR platform heading angle
look_angle = 360+platformheading + 90;                                     %calculate look angle
if look_angle > 360
    look_angle = look_angle-360;
end
clear platformheading

%% retrieve wind speed using BP neural network
%convert matrix to column vector
[H,L]=size(incidence);
incidence1=reshape(incidence,H*L,1);
ERA5_wd=reshape(era5_wd,H*L,1);
sigma_hh=reshape(sigma0L_hh2,H*L,1);
sigma0db=double(10*log10(sigma_hh));                                       %convert linear unit to dB unit

inc= incidence1*pi/180;
azimuth_angle=ERA5_wd-look_angle;                                          %calculate azimuth angle(azimuth angel=wind direction - look angle)
a=find(azimuth_angle(:)<0);
azimuth_angle(a,:)=azimuth_angle(a,:)+360;
clear a  

%calculate cos()cos(2)
azi_cos=cos(azimuth_angle*pi/180);
azi_cos2=cos(2*azimuth_angle*pi/180);

S1_data=[sigma0db azi_cos azi_cos2 inc];                                   %input data

load('WS_net.mat')                                                         %load BP neural network model
S1_datan= mapminmax('apply',S1_data',ps);                                  %input data normalization
S1_wsn=sim(net,S1_datan);                                                  %calculate wind speed
min=0; max=30;
S1_ws=(S1_wsn-(-1))*(max-min)/(1-(-1))+min;                                %output data denormalization
S1_ws=S1_ws';
S1_WS=reshape(S1_ws,H,L);                                                  %convert column vector to matrix

clear incidence1 ERA5_wd sigma_hh azi_cos azi_cos2 sigma0db azimuth_angle inc S1_datan S1_wsn

%% save the wind speed result
sswsInfo{1}=lon_resized;
sswsInfo{2}=lat_resized;
sswsInfo{3}=incidence;
sswsInfo{4}=sigma0L_hh2;
sswsInfo{5}=look_angle;
sswsInfo{6}=era5_ws;
sswsInfo{7}=era5_wd;
sswsInfo{8}=S1_WS;
save_path=['SSWS_' S_filepath(end-72:end-41) S_filepath(end-10:end-6) '_V1.0.mat'];
save(save_path,'sswsInfo');